'' =================================================================================================
''
''   File....... jm_ds18b20.spin
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2019-2025 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon.mcphalen@gmail.com
''   Started....
''   Updated.... 28 FEB 2025
''
'' =================================================================================================


con

  VERSION = 1_1_0


con { fixed io pins }

  RX_PGM   = 31  { I }                                          ' serial / programming
  TX_PGM   = 30  { O }

  SDA1     = 29  { I/O }                                        ' i2c / eeprom
  SCL1     = 28  { I/O }


con

  BUS_SHORT  = ow#BUS_SHORT
  BAD_RESET  = ow#BAD_RESET
  GOOD_RESET = ow#GOOD_RESET
  NO_DEVICE  = ow#NO_DEVICE


con

  RD_ROM     = $33                                              ' 1W ROM commands
  MATCH_ROM  = $55
  SKIP_ROM   = $CC
  ALARM_SRCH = $EC

  CVRT_TEMP  = $44                                              ' DS18xxx commands
  WR_SPAD    = $4E
  RD_SPAD    = $BE
  COPY_SPAD  = $48
  RD_EE      = $B8
  RD_POWER   = $B4


obj

  ow : "jm_1-wire"                                              ' 1-Wire driver


pub null

'' This is not an application


pub start(pin) : result

'' Start 1-Wire connection on pin

  return ow.start(pin)


pub reset : result

'' Resets 1-Wire bus; returns bus status
''
''   %00 = bus short
''   %01 = bad response; possible interference on bus
''   %10 = good bus & presence detection
''   %11 = no device

  return ow.reset


pub rd_bit : result

'' Read one bit from 1-Wire bus
'' -- use to monitor busy status

  return ow.rd_bit


pub read_sn(p_sn)

'' Reads serial number from 1W device
'' -- stores in array at p_sn
'' -- only connect one device at a time for this method

  ow.reset
  ow.write(RD_ROM)
  repeat 8
    byte[p_sn++] := ow.read


pub read_tc : tmpC

'' Reads temperature from DS18b20
'' -- skips id, so only one device can be connected
'' -- returns degrees C in 0.0001 degree units

  ow.reset
  ow.write(SKIP_ROM)                                            ' no serial number needed
  ow.write(CVRT_TEMP)                                           ' start temp conversion
  repeat                                                        ' let conversion finish
    tmpC := ow.rd_bit
  until (tmpC == 1)
  ow.reset
  ow.write(SKIP_ROM)
  ow.write(RD_SPAD)                                             ' read scratchpad
  tmpC.byte[0] := ow.read                                       ' lsb of temp
  tmpC.byte[1] := ow.read                                       ' msb of temp
  tmpC := ~~tmpC * 625                                          ' extend sign, 0.0001 units


pub c2f(tc) : tmpF

'' Converts Celcius to Fahrenheight
'' -- tc in 0.0001C units

  tmpF := tc * 9 / 5 + 32_0000                                   ' convert to Fahrenheit


pub read_tf : tmpF

'' Reads temperature from DS18b20
'' -- skips id, so only one device can be connected
'' -- returns degrees F in 0.0001 degree units

  return c2f(read_tc)


pub read_tca(p_sn) : tmpC | idx

'' Reads temperature from DS18b20 at specific address
'' -- returns degrees C in 0.0001 degree units
'' -- p_sn is pointer to byte array with serial #

  ow.reset
  ow.write(MATCH_ROM)                                           ' use serial #
  repeat idx from 0 to 7                                        ' send serial #
    ow.write(byte[p_sn][idx])
  ow.write(CVRT_TEMP)                                           ' start conversion
  repeat                                                        ' let conversion finish
    tmpC := ow.rd_bit
  until (tmpC == 1)
  ow.reset
  ow.write(MATCH_ROM)
  repeat idx from 0 to 7
    ow.write(byte[p_sn][idx])
  ow.write(RD_SPAD)                                             ' read scratchpad
  tmpC.byte[0] := ow.read                                       ' lsb of temp
  tmpC.byte[1] := ow.read                                       ' msb of temp
  tmpC := ~~tmpC * 625                                          ' extend sign, 0.0001 units


con { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}