'' =================================================================================================
''
''   File....... jm_ds18b20_demo.spin
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               -- Copyright (c) 2019-2025 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon.mcphalen@gmail.com
''   Started....
''   Updated.... 04 NOV 2024
''
'' =================================================================================================


con

  VERSION = 1_0_0


con { timing }

  _xinfreq = 5_000_000                                          ' use 5MHz crystal
  _clkmode = xtal1 + pll16x                                     ' x16 = 80MHz

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us


con { terminal }

  BR_TERM  = 115_200                                            ' terminal baud rate


con { fixed io pins }

  RX_PGM   = 31  { I }                                          ' serial / programming
  TX_PGM   = 30  { O }

  SDA1     = 29  { I/O }                                        ' i2c / eeprom
  SCL1     = 28  { I/O }

  LED27    = 27  { O }                                          ' LEDs on PAB and FLiP
  LED26    = 26  { O }


con { app io pins }

  OWX      = 16  { IO }                                         ' 1-Wire serial


con

  #true,  ON, OFF
  #false, NO, YES


obj

' main                                                          ' * master Spin cog
  time : "jm_time_80"                                           '   timing and delays (80MHz)
  io   : "jm_io"                                                '   simple io
  tsns : "jm_ds18b20"                                           '   1-Wire temperature sensor
  term : "jm_fullduplexserial"                                  ' * serial IO for terminal

' * uses cog when loaded


dat

  SN0           byte      $00, $00, $00, $00, $00, $00, $00, $00


var { globals }


pub main | status, t, idx, tmpc, tmpf

  setup
  wait_for_terminal(true, 250)

  status := tsns.reset                                          ' check for device

  if (status == %10)                                            ' good 1W reset
    tsns.read_sn(@SN0)                                          ' read the serial #
    term.tx(term#CLS)

    repeat idx from 0 to 7                                      ' display serial # bytes
      term.fstr1(string("%.2x "), SN0[idx])

    t := cnt                                                    ' sync timing loop
    repeat
      tmpc := tsns.read_tc                                      ' read the temperature
'     tmpc := tsns.read_tca(@SN0)                               ' read from specific address

      term.fstr0(string("\002\000\002TC = "))                   ' x/y = 0/2
      term.fstr1(string("%6.2f"), (tmpc+50)/100)                ' show tC, rounded to 1/100th degrees

      tmpf := tsns.c2f(tmpc)                                    ' convert to Fahrenheit

      term.fstr0(string("\002\000\003TF = "))
      term.fstr1(string("%6.2f"), (tmpf+50)/100)                ' show tF, rounded to 1/100th degrees

      waitcnt(t += CLK_FREQ)                                    ' update once per second

  else
    term.tx(term#CLS)
    case status
      %00 : term.str(string("Bus Short"))
      %01 : term.str(string("Bus Error"))
      %11 : term.str(string("No Device"))

    repeat
      waitcnt(0)                                                ' wait until reset


pub setup

'' Setup IO and objects for application

  time.start                                                    ' setup timing & delays

  io.start(0, 0)                                                ' clear all pins (master cog)

  tsns.start(OWX)                                               ' connect to 1W bus

  term.tstart(BR_TERM)                                          ' start serial for terminal *


pub wait_for_terminal(clear, delay)

'' Wait for terminal to be open and key pressed

  term.rxflush
  term.rx
  if (clear)
    term.tx(term#CLS)
    time.pause(delay #> 0)


con { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}